<?php



namespace WpieApp\Core\Upgrade;

use WP_Error;
use WpieApp\Core\Helpers\Helper;
use WpieApp\Core\Utils\License;
use WpieApp\Core\Logger\Logger;
use WpieApp\Modules\Import\Controller\File\Downloader\Url\Downloader\File;
use WpieApp\Core\Office\Reader\Json\Reader;

defined( 'WPIE__FILE__' ) || exit;


class Core {

        
        const TRANSIENT = 'wpie_plugin_updates';

        
        private static $forceChecked = false;

        
        public static function init() {

                if ( !WPIE_PRO ) {
                        return;
                }

                // If the user cannot update plugins, stop processing here.
                if ( !current_user_can( 'update_plugins' ) ) {
                        return;
                }

                \add_filter( 'pre_set_site_transient_update_plugins', [ __CLASS__, 'updatePluginsFilter' ], 1000 );

                \add_filter( 'plugins_api', [ __CLASS__, 'pluginsApi' ], 10, 3 );

                if ( \is_admin() ) {

                        \add_action( 'in_plugin_update_message-' . WPIE_BASENAME, [ __CLASS__, 'addNotice' ] );
                }
        }

        
        public static function updatePluginsFilter( $transient ) {

                // If no update object exists, bail to prevent errors.
                if ( empty( $transient ) || !is_object( $transient ) || !isset( $transient->response ) ) {
                        return $transient;
                }

                $forceCheck = (!self::$forceChecked) ? \absint( Helper::_get( 'force-check', 0 ) ) === 1 : false;

                self::$forceChecked = true;

                $data = self::getUpdates( $forceCheck );

                if ( false !== $data ) {
                        $transient->response[ WPIE_BASENAME ] = $data;
                }

                return $transient;
        }

        
        public static function getUpdates( $forceCheck ) {

                $update = self::getTransient( $forceCheck );

                if ( \is_wp_error( $update ) ) {
                        Logger::error( $update->get_error_message() );
                        return false;
                }

                if ( !is_object( $update ) ) {
                        return false;
                }

                $data = new \stdClass();

                $data->slug = basename( WPIE_BASENAME, '.php' );

                $data->plugin = WPIE_BASENAME;

                $data->new_version = isset( $update->version ) ? $update->version : "";

                $data->tested = isset( $update->tested ) ? $update->tested : "";

                $data->url = isset( $update->homepage ) ? $update->homepage : "";

                $data->icons = ( array ) $update->icons;

                $data->banners = ( array ) $update->banners;

                $data->package = "";

                $license = License::get();

                if ( $license ) {
                        $data->package = \add_query_arg(
                                [
                                        'vlm_api_action' => "update_package",
                                        'plugin'         => $data->slug,
                                        'license'        => base64_encode( $license ),
                                        'wp_url'         => \home_url(),
                                ],
                                $update->update_url );
                }

                $data->download_link = $data->package;

                return $data;
        }

        
        public static function pluginsApi( $api, $action = '', $args = null ) {

                $plugin = ( 'plugin_information' === $action ) && isset( $args->slug ) && ( WPIE_BASENAME === $args->slug );

                // If our plugin matches the request, set our own plugin data, else return the default response.
                if ( $plugin ) {
                        return self::setPluginsApi( $api );
                }

                return $api;
        }

        
        public static function setPluginsApi( $api ) {

                $data = self::getTransient();

                if ( \is_wp_error( $data ) ) {
                        Logger::error( $data->get_error_message() );
                        return false;
                }
                if ( !is_object( $data ) ) {
                        return $api;
                }

                $data->sections = isset( $data->sections ) ? ( array ) $data->sections : [];

                $data->icons = isset( $data->icons ) ? ( array ) $data->icons : [];

                $data->banners = isset( $data->banners ) ? ( array ) $data->banners : [];

                return $data;
        }

        private static function getTransient( $forceCheck = false ) {

                if ( !$forceCheck ) {

                        $transient = \get_transient( self::TRANSIENT );

                        if ( !(\is_object( $transient ) && isset( $transient->version ) && version_compare( WPIE_VERSION, $transient->version, '=' ) ) ) {
                                return $transient;
                        }
                }

                $data = self::downloadData();

                if ( \is_wp_error( $data ) ) {
                        return $data;
                }

                if ( !is_object( $data ) || !isset( $data->version ) ) {
                        return false;
                }

                if ( version_compare( WPIE_VERSION, $data->version, '<' ) ) {

                        \set_transient( self::TRANSIENT, $data, (12 * HOUR_IN_SECONDS ) );

                        return $data;
                }

                self::refreshTransient();

                return false;
        }

        
        public static function refreshTransient() {

                \delete_site_transient( 'update_plugins' );

                \delete_transient( self::TRANSIENT );
        }

        
        private static function downloadData() {

                $slug = substr( WPIE_BASENAME, 0, -4 );

                $url = WPIE_API_URL . 'items/' . $slug . '.json';

                $uri = new File( $url, [ 'json' ] );

                $fileInfo = $uri->download();

                if ( \is_wp_error( $fileInfo ) ) {
                        return new WP_Error( 'wpieError', __( "File download failed", 'wp-import-export' ) . ' : ' . $fileInfo->get_error_message() );
                }

                $tempPath = (is_array( $fileInfo ) && isset( $fileInfo[ 'path' ] )) ? $fileInfo[ 'path' ] : "";
                $fileName = (is_array( $fileInfo ) && isset( $fileInfo[ 'name' ] )) ? $fileInfo[ 'name' ] : "";

                $path = Helper::pathJoin( \dirname( $tempPath ), $fileName );

                if ( !Helper::duplicate( $tempPath, $path ) ) {
                        return new WP_Error( 'wpie_import_error', __( 'Failed to Copy Downloaded file', 'wp-import-export' ) );
                }

                $data = Reader::read( $path, false );

                Helper::deleteFile( [ $path, $tempPath ] );

                return $data;
        }

        
        public static function addNotice() {

                if ( License::isActive() ) {
                        return;
                }

                /* translators: %s: License Page URL */
                echo \wp_kses(
                        '<br /><strong>' .
                        \sprintf( __( 'To enable updates, please enter your purchase code on the <a href="%s">License</a> page', 'wp-import-export' ), \esc_url( Helper::adminUrl( 'wpie-license' ) ) ) .
                        '</strong>', Helper::ksesAllowedTags( [ 'br' => [] ] )
                );
        }
}
